#include "eiciel_add_acl_window.hpp"
#include "config.hpp"

EicielAddACLWindow::EicielAddACLWindow()
	: ACLusuari("Usuari"),
	ACLgrup("Grup"),
	aclDefault("ACL per defecte"),
	afegirParticipant(Gtk::Stock::ADD),
	eliminarParticipant(Gtk::Stock::REMOVE),
	permisLectura("Lectura"),
	permisEscriptura("Escriptura"),
	permisExecucio("Execucio"),
	numParticipantsAfegits(0)
{
	set_title("Afegir ACL");

	set_border_width(4);
	
	Gtk::RadioButton::Group tipusACL = ACLusuari.get_group();
	ACLgrup.set_group(tipusACL);

	marcTipusACL.set_label("Tipus ACL");
	tipusACLBox.add(ACLusuari);
	tipusACLBox.add(ACLgrup);
	tipusACLBox.add(aclDefault);
	marcTipusACL.add(tipusACLBox);

	zonaPrincipal.add(marcTipusACL);

	get_vbox()->set_spacing(4);
	get_vbox()->add(zonaPrincipal);

	refVistaPartic = Gtk::ListStore::create(modelPartic);
	refVistaParticAfegits = Gtk::ListStore::create(modelPartic);

	iconaUsuariACL = Gdk::Pixbuf::create_from_file(DATADIR "/img/usuari-acl.png");
	iconaGrupACL = Gdk::Pixbuf::create_from_file(DATADIR "/img/grup-acl.png");

	vistaPartic.set_model(refVistaPartic);
	vistaPartic.append_column("", modelPartic.iconeta);
	vistaPartic.append_column("Participant", modelPartic.nomParticipant);

	vistaParticAfegits.set_model(refVistaParticAfegits);
	vistaParticAfegits.append_column("", modelPartic.iconeta);
	vistaParticAfegits.append_column("Participant", modelPartic.nomParticipant);

	vistaPartic.set_size_request(-1, 100);
	vistaParticAfegits.set_size_request(-1, 100);

	botonsParticipants.add(afegirParticipant);
	botonsParticipants.add(eliminarParticipant);
	
	scrollVistaPartic.add(vistaPartic);
	scrollVistaPartic.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);

	zonaParticipants.add(scrollVistaPartic);
	zonaParticipants.add(botonsParticipants);
	scrollVistaParticAfegits.add(vistaParticAfegits);
	scrollVistaParticAfegits.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);

	zonaParticipants.add(scrollVistaParticAfegits);
	zonaParticipants.set_spacing(4);

	marcParticip.add(zonaParticipants);
	marcParticip.set_label("Escull els participants");

	zonaPrincipal.add(marcParticip);

	botonsPermisos.add(permisLectura);
	botonsPermisos.add(permisEscriptura);
	botonsPermisos.add(permisExecucio);
	botonsPermisos.set_spacing(4);

	marcPermisos.add(botonsPermisos);
	zonaPrincipal.add(marcPermisos);
	marcPermisos.set_label("Permisos");
	
	botoAcceptar = add_button(Gtk::Stock::OK, EicielAddACLWindow::ACCEPTAR);
	botoAcceptar->set_sensitive(false);
	add_button(Gtk::Stock::CANCEL, EicielAddACLWindow::CANCELAR);

	// Callbacks
	ACLusuari.signal_clicked().connect(
			SigC::slot(*this, &EicielAddACLWindow::mostrarUsuaris)
			);

	ACLgrup.signal_clicked().connect(
			SigC::slot(*this, &EicielAddACLWindow::mostrarGrups)
			);

	afegirParticipant.signal_clicked().connect(
			SigC::slot(*this, &EicielAddACLWindow::afegeixParticipant)
			);

	eliminarParticipant.signal_clicked().connect(
			SigC::slot(*this, &EicielAddACLWindow::eliminaParticipant)
			);

	Glib::RefPtr<Gtk::TreeSelection> refLlistaSel = vistaPartic.get_selection();
	refLlistaSel->signal_changed().connect(
			SigC::slot(*this, &EicielAddACLWindow::canviDeSeleccioPartic)
			);

	refVistaPartic->set_default_sort_func(
			SigC::slot(*this, &EicielAddACLWindow::ordenaParticipants)
			);
	refVistaPartic->set_sort_column_id(Gtk::TreeSortable::DEFAULT_SORT_COLUMN_ID, Gtk::SORT_ASCENDING);

	refVistaParticAfegits->set_default_sort_func(
			SigC::slot(*this, &EicielAddACLWindow::ordenaParticipants)
			);
	refVistaParticAfegits->set_sort_column_id(Gtk::TreeSortable::DEFAULT_SORT_COLUMN_ID, Gtk::SORT_ASCENDING);

	refLlistaSel = vistaParticAfegits.get_selection();
	refLlistaSel->signal_changed().connect(
			SigC::slot(*this, &EicielAddACLWindow::canviDeSeleccioParticAfegit)
			);
	
	show_all_children();
}

void EicielAddACLWindow::admetDefault(bool b)
{
	aclDefault.set_sensitive(b);
}

void EicielAddACLWindow::llistatUsuaris(vector<string>& llistat)
{
	llUsuaris = llistat;
}

void EicielAddACLWindow::llistatGrups(vector<string>& llistat)
{
	llGrups = llistat;
}

void EicielAddACLWindow::mostrarUsuaris()
{
	mostrarParticipants(llUsuaris, iconaUsuariACL);
}

void EicielAddACLWindow::mostrarGrups()
{
	mostrarParticipants(llGrups, iconaGrupACL);
}

void EicielAddACLWindow::mostrarParticipants(vector<string>& v, Glib::RefPtr<Gdk::Pixbuf> icona)
{
	refVistaPartic->clear();
	refVistaParticAfegits->clear();
	Gtk::TreeModel::iterator iter;
	
	for (vector<string>::iterator i = v.begin(); i != v.end(); i++)
	{
		iter = refVistaPartic->append();
		Gtk::TreeModel::Row row(*iter);
		row[modelPartic.iconeta] = icona;
		row[modelPartic.nomParticipant] = Glib::locale_to_utf8(*i);
	}
	afegirParticipant.set_sensitive(false);
	eliminarParticipant.set_sensitive(false);
	botoAcceptar->set_sensitive(false);
}

void EicielAddACLWindow::canviDeSeleccioPartic()
{
	Glib::RefPtr<Gtk::TreeSelection> referenciaSeleccioLlista = vistaPartic.get_selection();
	Gtk::TreeModel::iterator iter = referenciaSeleccioLlista->get_selected();
	if (!iter)
	{
		afegirParticipant.set_sensitive(false);
	}
	else
	{
		afegirParticipant.set_sensitive(true);
	}
}

void EicielAddACLWindow::canviDeSeleccioParticAfegit()
{
	Glib::RefPtr<Gtk::TreeSelection> referenciaSeleccioLlista = vistaParticAfegits.get_selection();
	Gtk::TreeModel::iterator iter = referenciaSeleccioLlista->get_selected();
	if (!iter)
	{
		eliminarParticipant.set_sensitive(false);
	}
	else
	{
		eliminarParticipant.set_sensitive(true);
	}
}

void EicielAddACLWindow::afegeixParticipant()
{
	Glib::RefPtr<Gtk::TreeSelection> referenciaSeleccioLlista = vistaPartic.get_selection();
	Gtk::TreeModel::iterator iter = referenciaSeleccioLlista->get_selected();
	if (iter)
	{
		Gtk::TreeModel::iterator iter2 = refVistaParticAfegits->append();
		Gtk::TreeModel::Row row_vella(*iter);
		Gtk::TreeModel::Row row_nova(*iter2);
		Glib::RefPtr<Gdk::Pixbuf> icona = row_vella[modelPartic.iconeta];
		Glib::ustring cadena = row_vella[modelPartic.nomParticipant];
		row_nova[modelPartic.iconeta] = icona;
		row_nova[modelPartic.nomParticipant] = cadena;
		refVistaPartic->erase(iter);
		numParticipantsAfegits++;

		if (numParticipantsAfegits == 1)
		{
			botoAcceptar->set_sensitive(true);
		}
	}
}

void EicielAddACLWindow::eliminaParticipant()
{
	Glib::RefPtr<Gtk::TreeSelection> referenciaSeleccioLlista = vistaParticAfegits.get_selection();
	Gtk::TreeModel::iterator iter = referenciaSeleccioLlista->get_selected();
	if (iter)
	{
		Gtk::TreeModel::iterator iter2 = refVistaPartic->append();
		Gtk::TreeModel::Row row_vella(*iter);
		Gtk::TreeModel::Row row_nova(*iter2);
		Glib::RefPtr<Gdk::Pixbuf> icona = row_vella[modelPartic.iconeta];
		Glib::ustring cadena = row_vella[modelPartic.nomParticipant];
		row_nova[modelPartic.iconeta] = icona;
		row_nova[modelPartic.nomParticipant] = cadena;
		refVistaParticAfegits->erase(iter);
		numParticipantsAfegits--;

		if (numParticipantsAfegits == 0)
		{
			botoAcceptar->set_sensitive(false);
		}
	}
}

int EicielAddACLWindow::ordenaParticipants(const Gtk::TreeModel::iterator& i, const Gtk::TreeModel::iterator& j)
{
	Gtk::TreeModel::Row ri(*i);
	Gtk::TreeModel::Row rj(*j);

	if (ri[modelPartic.nomParticipant] == rj[modelPartic.nomParticipant]) return 0;
	else if (ri[modelPartic.nomParticipant] < rj[modelPartic.nomParticipant]) return -1;
	else return 1;
}

TipusElement EicielAddACLWindow::tipusParticipant()
{
	if (aclDefault.get_active())
	{
		if (ACLusuari.get_active())
		{
			return DEFAULT_ACL_USUARI;
		}
		else
		{
			return DEFAULT_ACL_GRUP;
		}
	}
	else
	{
		if (ACLusuari.get_active())
		{
			return ACL_USUARI;
		}
		else
		{
			return ACL_GRUP;
		}
	}
}

vector<string> EicielAddACLWindow::llistaParticipants()
{
	vector<string> v;

	for (Gtk::TreeModel::iterator i = refVistaParticAfegits->children().begin();
			i != refVistaParticAfegits->children().end(); i++)
	{
		Gtk::TreeModel::Row row(*i);
		v.push_back(Glib::ustring(row[modelPartic.nomParticipant]));
	}
	
	return v;
}

bool EicielAddACLWindow::lectura()
{
	return permisLectura.get_active();
}

bool EicielAddACLWindow::escriptura()
{
	return permisEscriptura.get_active();
}

bool EicielAddACLWindow::execucio()
{
	return permisExecucio.get_active();
}
