#include <iostream>
#include <pwd.h>
#include <grp.h>
#include "eiciel_main_controler.hpp"
#include "tipus_element_acl.hpp"
#include "config.hpp"

using namespace std;

EicielMainControler::EicielMainControler() 
	: gestorACL(NULL) { }

void EicielMainControler::oberturaFitxer()
{
	Gtk::FileSelection dialog("Escolliu un fitxer o carpeta");
	dialog.set_transient_for(*finestra);
	int result = dialog.run();
	switch (result)
	{
		case(Gtk::RESPONSE_OK):
			{
				// cout << "S'ha escollit un fitxer" << endl;
				// cout << "Fitxer escollit: " << dialog.get_filename() << endl;
				try
				{
					GestorACL* nouGestor;
					nouGestor = new GestorACL(dialog.get_filename());

					delete gestorACL;
					gestorACL = nouGestor;
					actualitzarLlistaACL();
					finestra->establirNomFitxer(dialog.get_filename());
				}
				catch (GestorACLException e)
				{
					Gtk::MessageDialog missatge("No s'ha pogut obrir el fitxer. " + e.getMessage(), 
							Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK, true);
					missatge.run();
				}

				break;
			}
		case(Gtk::RESPONSE_CANCEL):
			{
				// cout << "Anullat" << endl;
				break;
			}
		default:
			{
				// cout << "Boto inesperat" << endl;
				break;
			}
	}
}

void EicielMainControler::actualitzarLlistaACL()
{
	actualitzantFinestra = true;
	finestra->buidaLlistaACL();
	// L'usuari
	permisos_t perms = gestorACL->donaUsuari();
	vector<entrada_acl> vACL;
	finestra->afegeixNoSeleccionable("Usuari", perms.lectura,
			perms.escriptura, perms.execucio, USUARI);
	vACL = gestorACL->donaACLAccessUsuari();
	for (vector<entrada_acl>::iterator i = vACL.begin(); 
			i != vACL.end(); i++)
	{
		finestra->afegeixSeleccionable(i->nom, i->lectura, 
				i->escriptura, i->execucio, ACL_USUARI);
	}

	perms = gestorACL->donaGrup();
	finestra->afegeixNoSeleccionable("Grup", perms.lectura,
			perms.escriptura, perms.execucio, GRUP);

	vACL = gestorACL->donaACLAccessGrup();
	for (vector<entrada_acl>::iterator i = vACL.begin(); 
			i != vACL.end(); i++)
	{
		finestra->afegeixSeleccionable(i->nom, i->lectura, 
				i->escriptura, i->execucio, ACL_GRUP);
	}

	if (gestorACL->teMascara())
	{
		perms = gestorACL->donaMascara();
		finestra->afegeixNoSeleccionable("Mscara", perms.lectura,
				perms.escriptura, perms.execucio, MASCARA);
	}

	perms = gestorACL->donaAltres();
	finestra->afegeixNoSeleccionable("Altres", perms.lectura,
			perms.escriptura, perms.execucio, ALTRES);

	finestra->activarBotoDefaultACL(gestorACL->esDirectori());
	finestra->hiHaDefaultACL(false);

	if (gestorACL->esDirectori())
	{
		bool hiHaACLDefault = false;
		if (gestorACL->teDefaultUsuari())
		{
			perms = gestorACL->donaDefaultUsuari();
			finestra->afegeixNoSeleccionable("Usuari per defecte", perms.lectura,
					perms.escriptura, perms.execucio, DEFAULT_USUARI);
			hiHaACLDefault = true;
		}

		vACL = gestorACL->donaACLDefaultUsuari();

		hiHaACLDefault |= (vACL.size() > 0);
		for (vector<entrada_acl>::iterator i = vACL.begin(); 
				i != vACL.end(); i++)
		{
			finestra->afegeixSeleccionable(i->nom, i->lectura, 
					i->escriptura, i->execucio, DEFAULT_ACL_USUARI);
		}


		if (gestorACL->teDefaultGrup())
		{
			perms = gestorACL->donaDefaultGrup();
			finestra->afegeixNoSeleccionable("Grup per defecte", perms.lectura,
					perms.escriptura, perms.execucio, DEFAULT_GRUP);
			hiHaACLDefault |= true;
		}

		vACL = gestorACL->donaACLDefaultGrup();

		hiHaACLDefault |= (vACL.size() > 0);
		for (vector<entrada_acl>::iterator i = vACL.begin(); 
				i != vACL.end(); i++)
		{
			finestra->afegeixSeleccionable(i->nom, i->lectura, 
					i->escriptura, i->execucio, DEFAULT_ACL_GRUP);
		}

		if (gestorACL->teDefaultMascara())
		{
			perms = gestorACL->donaDefaultMascara();
			finestra->afegeixNoSeleccionable("Mscara per defecte", perms.lectura,
					perms.escriptura, perms.execucio, DEFAULT_MASCARA);
			hiHaACLDefault |= true;
		}

		if (gestorACL->teDefaultAltres())
		{
			perms = gestorACL->donaDefaultAltres();
			finestra->afegeixNoSeleccionable("Altres per defecte", perms.lectura,
					perms.escriptura, perms.execucio, DEFAULT_ALTRES);
			hiHaACLDefault |= true;
		}
		finestra->hiHaDefaultACL(hiHaACLDefault);
	}
	actualitzantFinestra = false;
}

void EicielMainControler::sortidaAplicacio()
{
	Gtk::Main::quit();
}

void EicielMainControler::afegirACL()
{
	EicielAddACLWindow addACLWindow;
	vector<string> llistatUsuaris;
	vector<string> llistatGrups;
	generaLlistaUsuaris(llistatUsuaris);
	generaLlistaGrups(llistatGrups);
	addACLWindow.llistatUsuaris(llistatUsuaris);
	addACLWindow.llistatGrups(llistatGrups);
	addACLWindow.admetDefault(gestorACL->esDirectori());
	addACLWindow.mostrarUsuaris();
	int resultat = addACLWindow.run();

	if (resultat == EicielAddACLWindow::ACCEPTAR)
	{
		vector<string> llistaParticipants = addACLWindow.llistaParticipants();
		TipusElement e = addACLWindow.tipusParticipant();

		permisos_t p(addACLWindow.lectura(), addACLWindow.escriptura(), addACLWindow.execucio());

		for (vector<string>::iterator i = llistaParticipants.begin(); i != llistaParticipants.end();
				i++)
		{
			afegirEntradaACL(*i, e, p);
		}
	}
	actualitzarLlistaACL();
}

void EicielMainControler::afegirEntradaACL(string& s, TipusElement e, permisos_t& p)
{
	switch(e)
	{
		case ACL_USUARI :
			{
				gestorACL->modificarACLUsuari(s, p);
				break;
			}
		case ACL_GRUP :
			{
				gestorACL->modificarACLGrup(s, p);
				break;
			}
		case DEFAULT_ACL_USUARI :
			{
				gestorACL->modificarDefaultACLUsuari(s, p);
				break;
			}
		case DEFAULT_ACL_GRUP :
			{
				gestorACL->modificarDefaultACLGrup(s, p);
				break;
			}
		default:
			break;
	}
}

void EicielMainControler::generaLlistaUsuaris(vector<string>& llistat)
{
	llistat.clear();
	struct passwd* u;
	setpwent();
	while((u = getpwent()) != NULL)
	{
		llistat.push_back(u->pw_name);
	}
	endpwent();
}

void EicielMainControler::generaLlistaGrups(vector<string>& llistat)
{
	llistat.clear();
	struct group* g;
	setgrent();
	while ((g = getgrent()) != NULL)
	{
		llistat.push_back(g->gr_name);
	}
	endgrent();
}

void EicielMainControler::eliminarACL(string nomEntrada, TipusElement e)
{
	bool actualitzat = true;
	switch(e)
	{
		case ACL_USUARI :
			{
				gestorACL->eliminarACLUsuari(nomEntrada);
				break;
			}
		case ACL_GRUP :
			{
				gestorACL->eliminarACLGrup(nomEntrada);
				break;
			}
		case DEFAULT_ACL_USUARI :
			{
				gestorACL->eliminarDefaultACLUsuari(nomEntrada);
				break;
			}
		case DEFAULT_ACL_GRUP :
			{
				gestorACL->eliminarDefaultACLGrup(nomEntrada);
				break;
			}
		default:
			actualitzat = false;
			break;
	}

	if (actualitzat)
	{
		actualitzarLlistaACL();
	}
}

void EicielMainControler::actualitzaEntradaACL(TipusElement e, string s, 
		bool lectura, bool escriptura, bool execucio)
{
	permisos_t p(lectura, escriptura, execucio);

	switch(e)
	{
		case MASCARA :
			{
				gestorACL->modificarMascara(p);
				break;
			}
		case DEFAULT_MASCARA :
			{
				gestorACL->modificarMascaraDefault(p);
				break;
			}
		case USUARI :
			{
				gestorACL->modificarPermisosPropietari(p);
				break;
			}
		case GRUP :
			{
				gestorACL->modificarPermisosGrup(p);
				break;
			}
		case ALTRES :
			{
				gestorACL->modificarPermisosAltres(p);
				break;
			}
		case DEFAULT_USUARI :
			{
				gestorACL->modificarPermisosDefaultPropietari(p);
				break;
			}
		case DEFAULT_GRUP :
			{
				gestorACL->modificarPermisosDefaultGrup(p);
				break;
			}
		case DEFAULT_ALTRES :
			{
				gestorACL->modificarPermisosDefaultAltres(p);
				break;
			}
		case ACL_USUARI :
			{
				gestorACL->modificarACLUsuari(s, p);
				break;
			}
		case ACL_GRUP :
			{
				gestorACL->modificarACLGrup(s, p);
				break;
			}
		case DEFAULT_ACL_USUARI :
			{
				gestorACL->modificarDefaultACLUsuari(s, p);
				break;
			}
		case DEFAULT_ACL_GRUP :
			{
				gestorACL->modificarDefaultACLGrup(s, p);
				break;
			}
		default:
			break;
	}
}

void EicielMainControler::canviACLDefault()
{
	if (actualitzantFinestra) return;

	if (!finestra->donaACLDefault())
	{
		// Estas segur q vols esborrar les entrades per defecte ?
		Glib::ustring s("Vols esborrar totes les entrades ACL per defecte?");
		Gtk::MessageDialog borrarACL(*finestra, s, 
				Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
		int resultat = borrarACL.run();
		if (resultat == Gtk::RESPONSE_YES)
		{
			gestorACL->buidarACLDefecte();
			actualitzarLlistaACL();
		}
	}
	else
	{
		gestorACL->creaACLDefecte();
		actualitzarLlistaACL();
	}
}
